from wiserep_api.api import get_response, get_target_response

def get_target_property(iau_name, property_name, verbose=False):
    """Obtains the target's properties from Wiserep.

    Properties: ``type``, ``redshift``, ``host``, ``coords``
    and ``coords_deg``.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    property_name : str or list
        Name of the target's property or multiple properties.
    verbose: bool, default 'False'
        If True, print some of the intermediate information

    Returns
    -------
    target_properties: str, float or list
        The values of the target's properties.
    """
    # target's webpage
    response = get_target_response(iau_name, verbose)
    if response is None:
        return None

    properties_dict = {'type':'Type',
                       'redshift':'Redshift',
                       'host':'Host Name',
                       'coords':'RA/DEC (J2000)',
                       'coords_deg':'RA/DEC (J2000)',
                       } 
    
    if isinstance(property_name, str):
        properties_list = [property_name]
    else:
        properties_list = property_name

    target_properties = []
    for property in properties_list:
        assert property in properties_dict.keys(), f"Not a valid property: '{property}'"
        prop_str = properties_dict[property]

        # search for the property
        if property=='coords':
            if prop_str not in response.text:
                prop_str = prop_str.replace('J2000', '')  # relatively new targets do not have coordinates epoch
            split_text = response.text.split(f'{prop_str}</span><b><div class="value">')
        elif property=='coords_deg':
            if prop_str not in response.text:
                prop_str = prop_str.replace('J2000', '')
            split_text = response.text.split(f'{prop_str}</span><b><div class="value">')
            if len(split_text)>1:
                split_text = split_text[1].split('div class="alter-value">')
        else:
            split_text = response.text.split(f'{prop_str}</span><div class="value"><b>')

        if len(split_text) > 1:
            target_property = split_text[1].split("<")[0]
        else:
            target_property = ''
            
        if property=='redshift' and len(target_property)>0:
            target_property = float(target_property)

        target_properties.append(target_property)

    if len(target_properties)==1:
        target_properties = target_properties[0]

    return target_properties

def get_target_class(iau_name, verbose=False):
    """Obtains the target's classification (type) from Wiserep.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    verbose: bool, default 'False'
        If True, print some of the intermediate information

    Returns
    -------
    target_class: str
        The target's classification. Returns 'Unknown' if not found.
    """
    # target's webpage
    response = get_target_response(iau_name, verbose)

    # search for classification
    split_text = response.text.split('Type</span><div class="value"><b>')
    if len(split_text) > 1:
        # look for classification under "Type" parameter
        target_class = split_text[1].split("<")[0]
        if target_class != "SN":
            return target_class

    # look for classifications in TNS reports at the bottom of the webpage
    table = response.text.split("\n <thead><tr>")[-1]
    simply_a_SN = False
    for row in table.split('<td class="cell-objtype_name">'):
        target_class = row.split("<")[0]
        if len(target_class) > 0 and target_class != "SN":
            return target_class

        if target_class == "SN":
            simply_a_SN = True

    if simply_a_SN is True:
        # Some objects just have the classification as "SN"
        return "SN"
    else:
        print(f"Target classification not found: {iau_name}")
        return "Unknown"



import requests


def get_response(url, verbose=False):
    """Obtains the response from a given Wiserep URL.

    Parameters
    ----------
    url: str
        Wiserep URL.
    verbose: bool, default 'False'
        Whether to print the errors.

    Returns
    -------
    response: requests.Response
        Response object.
    """
    # ID of your Bot:
    YOUR_BOT_ID = 1234
    # name of your Bot:
    YOUR_BOT_NAME = "My_Bot1"
    # API key of your Bot:
    api_key = "604d60d302f86eb38fd1407abe41d05b438043bd"

    headers = {
        "User-Agent": 'tns_marker{"tns_id":' + str(YOUR_BOT_ID) + ', "type":"bot",'
        ' "name":"' + YOUR_BOT_NAME + '"}'
    }

    http_errors = {
        304: "Error 304: Not Modified: There was no new data to return.",
        400: "Error 400: Bad Request: The request was invalid. "
        "An accompanying error message will explain why.",
        403: "Error 403: Forbidden: The request is understood, but it has "
        "been refused. An accompanying error message will explain why.",
        404: "Error 404: Not Found: The URI requested is invalid or the "
        "resource requested, such as a category, does not exists.",
        500: "Error 500: Internal Server Error: Something is broken.",
        503: "Error 503: Service Unavailable.",
    }

    ###############################################################
    response = requests.get(url, headers=headers)

    if response.status_code == 200:
        return response
    else:
        if verbose is True:
            print(http_errors[response.status_code], url)
        return None
    
def get_target_response(iau_name, verbose=False):
    """Obtains the response from a given target's Wiserep URL.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    verbose: bool, default 'False'
        Whether to print the errors.

    Returns
    -------
    response: requests.Response
        Response object.
    """
    target_url = f"https://www.wiserep.org/iauname/{iau_name}"
    response = get_response(target_url, verbose)
    if response is None:
        # try internal survey name
        target_url = f"https://www.wiserep.org/internal-name/{iau_name}"
        response = get_response(target_url, verbose)

        if response is None:
            return None
        
    return response


def _get_object_id(iau_name, verbose=False):
    """Obtains the objects's Wiserep ID.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    verbose: bool, default 'False'
        If True, print some of the intermediate information

    Returns
    -------
    obj_id: str
        The object's Wiserep ID. Returns 'Unknown' if not found
        or None if there is a problem of some other kind.
    """
    # look for the target ID in the search webpage
    wiserep_search_url = "https://www.wiserep.org/search?"
    search_name = iau_name.replace('+', '%2B')  # is this an html thing?
    target_search_url = wiserep_search_url + f'name={search_name}&name_like=1'
    response = get_response(target_search_url)
    if response is None:
        print(f'Could not load the webpage of {iau_name}')
        return None

    obj_id = None
    split_text = response.text.split('="Click to Object page">')
    for st in split_text:
        if f'{iau_name}<' in st:
            if 'href="/object/' in st:
                obj_id = st.split('href="/object/')[1].split('"')[0]
                break
            
    if verbose:
        print("Object ID (Wiserep):", obj_id)

    if obj_id is None:
        print(f"No target with this name found on Wiserep: {iau_name}")
    
    return obj_id



import os
import pandas as pd
from io import StringIO
from astropy.io import fits
from wiserep_api.api import get_response, get_target_response


def exclude_include(url, exclude=None, include=None):
    """Either includes or excludes the given URL according to the
    given patterns.

    Note: only one of the parameters ('exclude' or 'include') can be
    given at a time, not both

    Parameters
    ----------
    url : str
        URL string.
    exclude : list, optional
        Patterns to exclude URL.
    include : _type_, optional
        Patterns to include URL.

    Returns
    -------
    skip: bool
        Whether to exclude/include the given URL.
    """
    err_msg = "'exclude' and 'include' cannot be given at the same time!"
    assert exclude is None or include is None, err_msg

    if exclude is not None:
        skip = False
        for pattern in exclude:
            if pattern in url:
                skip = True

    if include is not None:
        skip = True
        for pattern in include:
            if pattern in url:
                skip = False

    if exclude is None and include is None:
        skip = False

    return skip


def download_target_spectra(
    iau_name, file_type=None, exclude=None, include=None, verbose=False
):
    """Downloads the target's spectra from Wiserep.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    file_type: str
        File format: either 'ascii' or 'fits'. By default,
        both formats are downloaded.
    exclude: list, default 'None'
        Files with the given string patterns are excluded.
        Cannot be given together with 'include'.
    include: list, default 'None'
        Files with the given string patterns are inxcluded.
        Cannot be given together with 'exclude'.
    verbose: bool, default 'False'
        If 'True', print some of the extra information.

    Returns
    -------
    target_class: str
        The target's classification. Returns 'Unknown' if not found.
    """
    if os.path.isdir("spectra") is False:
        os.mkdir("spectra")

    assert file_type in [None, "ascii", "fits"], "not a valide file type"

    # target's url
    response = get_target_response(iau_name, verbose)

    # search for spectra URLs
    # ASCII
    txt_urls = []
    split_text = response.text.split("asciifile=https%3A//")
    for st in split_text:
        url = st.split('"')[0]
        if (url not in txt_urls) and ("&amp" not in url) and ("DOCTYPE" not in url):
            txt_urls.append(url)
    if verbose is True:
        print(f"Found {len(txt_urls)} URLs with spectra (ASCII): {txt_urls}")
        
    # FITS
    fits_urls = []
    split_text = response.text.split("https://")
    for st in split_text:
        url = st.split('"')[0]
        if (url not in fits_urls) and (".fits" in url) and ("rel-file" not in url):
            fits_urls.append(url)
    if verbose is True:
        print(f"Found {len(fits_urls)} URLs with spectra (FITS): {fits_urls}")

    # retrieve table with spectra information
    spec_table = pd.read_html(StringIO(response.text), match='Spec. ID')[0]
    if isinstance(spec_table['Spec. ID'], pd.core.frame.DataFrame) is True:
        # from multi-index to the usual dataframe
        spec_dict = {col[0]:None for col in spec_table.columns}

        for col in spec_dict.keys():
            series = spec_table[col].copy()
            for sub_col in spec_table[col].columns:
                series = series[sub_col]
            spec_dict[col] = series.values

        spec_table = pd.DataFrame(spec_dict)
        # remove extra columns
        columns = [col for col in spec_table.columns if 'Unnamed' not in col]
        spec_table = spec_table[columns]
    
    # download ASCII spectra
    if file_type == "ascii" or file_type is None:
        ascii_files = []
        for url in txt_urls:
            # skip filesspec_df = spec_df[::2]  # every other row is just crap

            skip = exclude_include(url, exclude, include)
            if skip is True:
                if verbose is True:
                    print(f"Skipping {url}")
                continue

            # check url
            response = get_response("http://" + url)
            if response is None:
                print(f"Nothing found in {url}")

            # get spectrum
            basename = os.path.basename(url)
            obj_dir = os.path.join("spectra", iau_name)
            if os.path.isdir(obj_dir) is False:
                os.mkdir(obj_dir)
            outfile = os.path.join(obj_dir, basename)

            with open(outfile, "w") as file:
                file.write(response.text)

            if response.text.startswith("BITPIX") or response.text.startswith("SIMPLE"):
                # file includes header
                for i, line in enumerate(response.text.split("\n")):
                    split_line = line.split()
                    try:
                        _ = float(split_line[0])
                        _ = float(split_line[1])
                        skiprows = i
                        break
                    except:
                        continue
            else:
                skiprows = None
            spec_df = pd.read_csv(
                outfile,
                sep='\s+',
                names=["wave", "flux", "flux_err"],
                comment="#",
                skiprows=skiprows,
            )
            spec_df.to_csv(outfile, index=False)

            ascii_files.append(basename)
        # update table with the extracted files online
        spec_table = spec_table[spec_table['Spectrum ascii File'].isin(ascii_files)]

    # download FITS spectra
    if file_type == "fits" or file_type is None:
        fits_files = []
        for url in fits_urls:
            # skip files
            skip = exclude_include(url, exclude, include)
            if skip is True:
                if verbose is True:
                    print(f"Skipping {url}")
                continue

            # download file
            print(url)
            hdu = fits.open("http://" + url)

            basename = os.path.basename(url)
            obj_dir = os.path.join("spectra", iau_name)
            if os.path.isdir(obj_dir) is False:
                os.mkdir(obj_dir)
            outfile = os.path.join(obj_dir, basename)
            if os.path.isfile(outfile) is True:
                overwrite = True
            else:
                overwrite = False

            hdu.writeto(outfile, overwrite=overwrite, output_verify="ignore")

            fits_files.append(basename)
        # The fits files are not always available, so only update the table
        # according to these if the ascii files were not downloaded
        if 'ascii_files' not in locals():
            spec_table = spec_table[spec_table['Spectrum fits File'].isin(fits_files)]

    # save spectra information
    # if 'obj_dir' variable is not defined, it means that no spectrum was
    # downloaded
    if 'obj_dir' in locals():
        spec_file = os.path.join(obj_dir, 'downloaded_spectra_info.csv') 
        # remove crap | sort_index is to avoid warning
        spec_table = spec_table.drop(columns=['Select'], axis=1) 
        spec_table.to_csv(spec_file, index=False)



import os
import unittest
from wiserep_api import print_spectral_types, download_sn_list

if os.path.isfile("SNIa-CSM_wiserep.txt") is True:
    os.remove("SNIa-CSM_wiserep.txt")

class TestSearch(unittest.TestCase):
    def test_search(self):
        print_spectral_types()
        download_sn_list("SN Ia-CSM")

        assert os.path.isfile(
            "SNIa-CSM_wiserep.txt"
        ), "The downloaded list was not found"

if __name__ == "__main__":
    unittest.main()



import os
import glob
import shutil
import unittest
import numpy as np
import warnings
from astropy.utils.exceptions import AstropyWarning
from wiserep_api import download_target_spectra

if os.path.isdir('spectra') is True:
    shutil.rmtree('spectra')

class TestDownloadSpectra(unittest.TestCase):
    def test_download_spectra(self):
        target = "ASASSN-14jg"
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", AstropyWarning)
            download_target_spectra(target)
        files = glob.glob(os.path.join("spectra", target, "*"))

        txt_files = [file for file in files if ".fits" not in file and ".csv" not in file]
        err_msg = "Number of files does not match (ASCII)"
        np.testing.assert_equal(len(txt_files), 5, err_msg)

        fits_files = [file for file in files if ".fits" in file]
        err_msg = "Number of files does not match (FITS)"
        np.testing.assert_equal(len(fits_files), 3, err_msg)

    def test_exclude(self):
        target = "2017ixi"
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", AstropyWarning)
            download_target_spectra(target, exclude=["PESSTO"])
        files = glob.glob(os.path.join("spectra", target, "*"))

        txt_files = [file for file in files if ".fits" not in file and ".csv" not in file]
        err_msg = "There should be no files downloaded (ASCII)"
        np.testing.assert_equal(len(txt_files), 0, err_msg)

        fits_files = [file for file in files if ".fits" in file]
        err_msg = "There should be no files downloaded (FITS)"
        np.testing.assert_equal(len(fits_files), 0, err_msg)

    def test_include(self):
        target = "ASASSN-14hr"
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", AstropyWarning)
            download_target_spectra(target, include=["WiFeS"])
        files = glob.glob(os.path.join("spectra", target, "*"))

        txt_files = [file for file in files if ".fits" not in file and ".csv" not in file]
        err_msg = "Number of files does not match (ASCII)"
        np.testing.assert_equal(len(txt_files), 2, err_msg)

        fits_files = [file for file in files if ".fits" in file]
        err_msg = "Number of files does not match (FITS)"
        np.testing.assert_equal(len(fits_files), 2, err_msg)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from ._version import __version__

from .api import _get_object_id, get_target_response
from .properties import get_target_property, get_target_class
from .spectra import download_target_spectra
from .search import print_spectral_types, download_sn_list
from .snid import run_snid
